package net.wanji.web.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.AreaEnum;
import net.wanji.common.enums.ControlModeEnum;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.bo.AreaDetailBO;
import net.wanji.databus.bo.AreaIdBO;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossSectionMapper;
import net.wanji.databus.dao.mapper.RidInfoMapper;
import net.wanji.databus.dto.RunningPlanDTO;
import net.wanji.databus.po.AreaDetailPOExt;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.AreaDetailVO;
import net.wanji.databus.vo.AreaInfoVO;
import net.wanji.databus.vo.CrossIdAndNameVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.bo.AddOrUpdateAreaBO;
import net.wanji.web.bo.ControlModeBO;
import net.wanji.web.bo.PolygonBO;
import net.wanji.web.bo.SpecialServiceRouteBO;
import net.wanji.web.mapper.TBaseAreaCrossMapper;
import net.wanji.web.service.CrossManageService;
import net.wanji.web.service.scheme.impl.RunningPlanServiceImpl;
import net.wanji.web.vo.AreaTreeVO;
import net.wanji.web.vo.ControlModeVO;
import net.wanji.web.vo.ListAreaVO;
import net.wanji.web.vo.SelectCrossesByStartEndVO;
import net.wanji.web.vo.specialService.RouteElementVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/6/25 8:56
 */
@Service
@Slf4j
public class CrossManageServiceImpl implements CrossManageService {
    private final BaseCrossInfoMapper crossInfoMapper;
    private final BaseAreaInfoMapper baseAreaInfoMapper;
    private final SpecialServiceServiceImpl specialServiceService;
    private final TBaseAreaCrossMapper areaCrossMapper;
    private final BaseCrossSectionMapper crossSectionMapper;
    private final RunningPlanServiceImpl runningPlanServiceImpl;
    private final UtcFeignClients utcFeignClients;
    private final RidInfoMapper ridInfoMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    public CrossManageServiceImpl(BaseCrossInfoMapper crossInfoMapper, BaseAreaInfoMapper baseAreaInfoMapper,
                                  SpecialServiceServiceImpl specialServiceService, TBaseAreaCrossMapper areaCrossMapper, BaseCrossSectionMapper crossSectionMapper, RunningPlanServiceImpl runningPlanServiceImpl, @Qualifier("net.wanji.feign.service.UtcFeignClients") UtcFeignClients utcFeignClients, @Qualifier("ridInfoMapper") RidInfoMapper ridInfoMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.baseAreaInfoMapper = baseAreaInfoMapper;
        this.specialServiceService = specialServiceService;
        this.areaCrossMapper = areaCrossMapper;
        this.crossSectionMapper = crossSectionMapper;
        this.runningPlanServiceImpl = runningPlanServiceImpl;
        this.utcFeignClients = utcFeignClients;
        this.ridInfoMapper = ridInfoMapper;
    }

    @Override
    public List<CrossIdAndNameVO> selectCrossesByPolygon(PolygonBO polygonBO) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        String wkt = polygonBO.getWkt();
        List<BaseCrossInfoPO> crossList = crossInfoMapper.selectAll();

        for (BaseCrossInfoPO cross : crossList) {
            String location = cross.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String crossLonLat = lonLat[0] + "," + lonLat[1];
            Geometry wktGeo = GeometryUtil.str2Geometry(wkt);
            Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLat);
            // 判断点和多边形是否相交
            boolean isIntersect = wktGeo.intersects(crossGeo);
            if (isIntersect) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(cross.getId());
                crossIdAndNameVO.setCrossName(cross.getName());
                res.add(crossIdAndNameVO);
            }
        }

        return res;
    }

    @Override
    @Transactional
    public AreaIdBO addOrUpdateArea(AddOrUpdateAreaBO addOrUpdateAreaBO) {
        Integer type = addOrUpdateAreaBO.getType();
        Integer areaId = addOrUpdateAreaBO.getAreaId();
        Integer parentCode = addOrUpdateAreaBO.getParentCode();
        String areaName = addOrUpdateAreaBO.getAreaName();
        String roadName = addOrUpdateAreaBO.getRoadName();
        String remark = addOrUpdateAreaBO.getRemark();
        List<String> crossIdList = addOrUpdateAreaBO.getCrossIdList();
        String wkt = addOrUpdateAreaBO.getWkt();

        if (ObjectUtil.isNotEmpty(areaId) && 0 != areaId) {
            // 传ID为修改
            // 修改区域基础信息
            // 根据区域rid计算wkt
            List<String> wktList = calcWkt(crossIdList);
            wkt = wktList.toString();
            baseAreaInfoMapper.updateAreaNameAndRemark(areaName, remark, areaId, roadName, parentCode, wkt);
            // 修改区域关联路口
            areaCrossMapper.deleteByAreaId(areaId);
            for (String crossId : crossIdList) {
                areaCrossMapper.insertOne(crossId, areaId);
            }
            AreaIdBO areaIdBO = new AreaIdBO();
            areaIdBO.setAreaId(areaId);
            return areaIdBO;
        } else {
            // 不传ID为新增
            if (2 == type) { // 交警辖区
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                if (parentCode != null) {
                    baseAreaInfoPO.setParentCode(parentCode);
                } else {
                    baseAreaInfoPO.setParentCode(0);
                }
                baseAreaInfoPO.setRoadName("");
                // 根据区域rid计算wkt
                List<String> wktList = calcWkt(crossIdList);
                wkt = wktList.toString();
                AreaIdBO areaIdBO = handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
                return areaIdBO;
            } else if (6 == type) { // 道路
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                baseAreaInfoPO.setRoadName(roadName);
                baseAreaInfoPO.setParentCode(0);
                AreaIdBO areaIdBO = handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
                return areaIdBO;
            } else if (4 == type) { // 子区
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                baseAreaInfoPO.setRoadName("");
                baseAreaInfoPO.setParentCode(0);
                // 根据区域rid计算wkt
                List<String> wktList = calcWkt(crossIdList);
                wkt = wktList.toString();
                AreaIdBO areaIdBO = handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
                return areaIdBO;
            }
        }
        return null;
    }

    private List<String> calcWkt(List<String> crossIdList) {
        List<String> res = new ArrayList<>();

        List<RidInfoEntity> ridInfoPOList = ridInfoMapper.selectByCrossList(crossIdList);
        if (ridInfoPOList == null) {
            throw new RuntimeException("路网数据未包含区域内所有路口");
        }
        // 去除多边形范围外的路段
        List<RidInfoEntity> firstFilterRids = ridInfoPOList.stream()
                .filter(rid -> validRid(rid, crossIdList))
                .collect(Collectors.toList());

        // 验证路口数量
        Set<String> ridCrossIds = new HashSet<>();
        for (RidInfoEntity firstFilterRid : firstFilterRids) {
            String startCrossId = firstFilterRid.getStartCrossId();
            String endCrossId = firstFilterRid.getEndCrossId();
            ridCrossIds.add(startCrossId);
            ridCrossIds.add(endCrossId);
        }
        if (ridCrossIds.size() != crossIdList.size()) {
            throw new RuntimeException("路网数据未包含区域内所有路口");
        }

        for (RidInfoEntity firstFilterRid : firstFilterRids) {
            String wkt = firstFilterRid.getWkt();
            res.add(wkt);
        }
        return res;
    }

    private boolean validRid(RidInfoEntity rid, List<String> crossIdList) {
        String startCrossId = rid.getStartCrossId();
        String endCrossId = rid.getEndCrossId();
        return crossIdList.contains(startCrossId) && crossIdList.contains(endCrossId);
    }

    private AreaIdBO handleAreaInfoPO(Integer type, String areaName, String remark, List<String> crossIdList, String wkt,
                                      BaseAreaInfoPO baseAreaInfoPO) {
        baseAreaInfoPO.setCode(0);
        baseAreaInfoPO.setLocation("");
        baseAreaInfoPO.setName(areaName);
        baseAreaInfoPO.setType(type);
        baseAreaInfoPO.setPolylines(wkt);
        baseAreaInfoPO.setRemark(remark);
        baseAreaInfoMapper.insertOne(baseAreaInfoPO);
        Integer areaIdGenerated = baseAreaInfoPO.getId();
        for (String crossId : crossIdList) {
            areaCrossMapper.insertOne(crossId, areaIdGenerated);
        }
        AreaIdBO areaIdBO = new AreaIdBO();
        areaIdBO.setAreaId(areaIdGenerated);
        return areaIdBO;
    }

    @Override
    public SelectCrossesByStartEndVO selectCrossesByStartEnd(SpecialServiceRouteBO specialServiceRouteBO) {
        SelectCrossesByStartEndVO selectCrossesByStartEndVO = new SelectCrossesByStartEndVO();
        StringBuilder wkt = new StringBuilder();

        List<List<RouteElementVO>> twoRoutes = specialServiceService.specialServiceRoute(specialServiceRouteBO);;
        List<RouteElementVO> route = twoRoutes.get(0); // 选最优路线

        List<CrossIdAndNameVO> crossIdAndNameVOList = new ArrayList<>();
        for (int i = 0; i < route.size(); i++) {
            RouteElementVO routeElementVO = route.get(i);
            String wktPart = routeElementVO.getWkt();
            wkt.append(wktPart);
            CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
            crossIdAndNameVO.setCrossId(routeElementVO.getStartId());
            crossIdAndNameVO.setCrossName(routeElementVO.getStartName());
            crossIdAndNameVOList.add(crossIdAndNameVO);
            if (i == route.size() - 1) { // 最后一段
                CrossIdAndNameVO crossIdAndNameVOEnd = new CrossIdAndNameVO();
                crossIdAndNameVOEnd.setCrossId(routeElementVO.getEndId());
                crossIdAndNameVOEnd.setCrossName(routeElementVO.getEndName());
                crossIdAndNameVOList.add(crossIdAndNameVOEnd);
            }
        }

        selectCrossesByStartEndVO.setWkt(wkt.toString());
        selectCrossesByStartEndVO.setCrossIdAndNameVOList(crossIdAndNameVOList);
        return selectCrossesByStartEndVO;
    }

    @Override
    @Transactional
    public void deleteArea(AreaIdBO areaIdBO) {
        Integer areaId = areaIdBO.getAreaId();
        // 删除区域路口关系
        List<Integer> areaIds = baseAreaInfoMapper.selectIdsByParentCode(areaId);
        areaIds.add(areaId);
        areaCrossMapper.deleteByAreaIds(areaIds);
        // 删除区域基础信息
        baseAreaInfoMapper.deleteById(areaId);
    }

    @Override
    public ListAreaVO listArea() {
        ListAreaVO listAreaVO = new ListAreaVO();

        listAreaVO.setJuriList(buildJuriList());
        listAreaVO.setRoadList(buildRoadList());
        listAreaVO.setSubAreaList(buildSubAreaList());

        return listAreaVO;
    }

    @Override
    public PageInfo<AreaDetailVO> areaDetail(AreaDetailBO areaDetailBO) throws Exception {
        Integer areaId = areaDetailBO.getAreaId();
        Integer pageNum = areaDetailBO.getPageNum();
        Integer pageSize = areaDetailBO.getPageSize();

        // 可选参数
        String crossName = areaDetailBO.getCrossName();
        Integer isSignal = areaDetailBO.getIsSignal();
        Integer controlMode = areaDetailBO.getControlMode();
        Integer isStart = areaDetailBO.getIsStart();
        Integer isSend = areaDetailBO.getIsSend();

        PageHelper.startPage(pageNum, pageSize);
        List<AreaDetailPOExt> areaDetailPOExtList = new ArrayList<>();
        // 如果不传areaId或ID为0，则返回全部路口
        if (areaId == null || areaId == 0) {
            areaDetailPOExtList = baseAreaInfoMapper.selectAllJoinDetail(crossName, isSignal, isStart, isSend);
        } else {
            areaDetailPOExtList = baseAreaInfoMapper.selectJoinDetail(areaId, crossName, isSignal, isStart, isSend);
        }
        PageInfo<AreaDetailPOExt> extPageInfo = new PageInfo<>(areaDetailPOExtList);

        PageInfo<AreaDetailVO> voPageInfo = new PageInfo<>();
        BeanUtils.copyProperties(extPageInfo, voPageInfo);

        List<AreaDetailPOExt> extList = extPageInfo.getList();
        // 调用utc接口，填充信号机信息
        for (AreaDetailPOExt areaDetailPOExt : extList) {
            String crossId = areaDetailPOExt.getCrossId();
            CrossIdBO crossIdBO = new CrossIdBO();
            crossIdBO.setCrossId(crossId);
            JsonViewObject jsonViewObject = utcFeignClients.selectCrossInfoById(crossIdBO);
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                throw new Exception("远程调用UTC服务路口信息失败！");
            }
            Map content = (Map) jsonViewObject.getContent();
            if (content != null) {
                areaDetailPOExt.setCode((String) content.get("code"));
                areaDetailPOExt.setManufacturerId((Integer) content.get("manufacturerId"));
                areaDetailPOExt.setIp((String) content.get("ip"));
                areaDetailPOExt.setPort((Integer) content.get("port"));
                areaDetailPOExt.setLocation((String) content.get("location"));
                areaDetailPOExt.setVersion((String) content.get("version"));
                areaDetailPOExt.setModel((String) content.get("model"));
                String s = (String) content.get("installTime");
                Date parsed = DateUtil.parse(s, DateTimeFormatter.ofPattern(DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                areaDetailPOExt.setInstallTime(parsed);
                areaDetailPOExt.setNickName((String) content.get("nickName"));
            }

        }

        List<AreaDetailVO> voList = new ArrayList<>();
        for (AreaDetailPOExt areaDetailPOExt : extList) {
            AreaDetailVO areaDetailVO = new AreaDetailVO();
            areaDetailVO.setAreaId(areaDetailPOExt.getAreaId());
            areaDetailVO.setId(areaDetailPOExt.getCrossId());
            areaDetailVO.setInstallTime(areaDetailPOExt.getInstallTime());
            areaDetailVO.setName(areaDetailPOExt.getCrossName());
            areaDetailVO.setAreaName(areaDetailPOExt.getAreaName());
            areaDetailVO.setCode(areaDetailPOExt.getCode());
            areaDetailVO.setManufacturerId(areaDetailPOExt.getManufacturerId());
            areaDetailVO.setManufacturerNick(areaDetailPOExt.getNickName());
            areaDetailVO.setModel(areaDetailPOExt.getModel());
            String ip = areaDetailPOExt.getIp();
            Integer port = areaDetailPOExt.getPort();
            if (ip != null) {
                areaDetailVO.setIpPort(ip + ":" + port);
            }
            areaDetailVO.setIp(ip);
            areaDetailVO.setPort(port);
            areaDetailVO.setLocation(areaDetailPOExt.getLocation());
            areaDetailVO.setVersion(areaDetailPOExt.getVersion());
            areaDetailVO.setIsSignal(areaDetailPOExt.getIsSignal());
            areaDetailVO.setIsStart(areaDetailPOExt.getIsStart());
            areaDetailVO.setIsSend(areaDetailPOExt.getIsSend());

            // 获取控制模式
            Date datetime = new Date();
            String dateStr = sdf.format(datetime);
            String crossId = areaDetailPOExt.getCrossId();
            // 获取当前计划
            Integer planId = findPlanId(datetime, dateStr, crossId);
            // 当前时段
            CrossSectionPO currentCrossSection = null;
            if (planId != null) {
                List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossAndPlan(crossId, planId);
                // 获取当前时间整数字面量
                SimpleDateFormat formatter = new SimpleDateFormat("HHmm");
                String timeString = formatter.format(datetime);
                int nowTimeInt = Integer.parseInt(timeString);
                for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                    String startTime = crossSectionPO.getStartTime();
                    String endTime = crossSectionPO.getEndTime();
                    Integer startTimeInt = timeStrToInt(startTime);
                    Integer endTimeInt = timeStrToInt(endTime);
                    if (nowTimeInt <= endTimeInt && nowTimeInt >= startTimeInt) {
                        currentCrossSection = crossSectionPO;
                    }
                }
            }
            // 筛选控制模式
            if (controlMode != null) {
                if (currentCrossSection != null) {
                    Integer currentControlMode = currentCrossSection.getControlMode();
                    if (controlMode.equals(currentControlMode)) {
                        areaDetailVO.setControlMode(currentControlMode);
                        voList.add(areaDetailVO);
                    }
                }
            } else {
                if (currentCrossSection != null) {
                    Integer currentControlMode = currentCrossSection.getControlMode();
                    areaDetailVO.setControlMode(currentControlMode);
                }
                voList.add(areaDetailVO);
            }
        }
        voPageInfo.setList(voList);
        return voPageInfo;
    }

    @Override
    public List<ControlModeVO> controlModeMenu() {
        List<ControlModeVO> res = new ArrayList<>();

        for (ControlModeEnum value : ControlModeEnum.values()) {
            Integer code = value.getCode();
            String name = value.getName();
            ControlModeVO controlModeVO = new ControlModeVO();
            controlModeVO.setControlMode(code);
            controlModeVO.setControlModeStr(name);

            res.add(controlModeVO);
        }
        return res;
    }

    @Override
    @Transactional
    public void updateControlMode(ControlModeBO controlModeBO) {
        Integer controlMode = controlModeBO.getControlMode();
        String crossId = controlModeBO.getCrossId();
        // 获取当前时段ID
        Integer sectionId = findCurrentSection(crossId);
        if (sectionId != null && 0 != sectionId) {
            crossSectionMapper.updateControlMode(sectionId, controlMode);
        } else {
            throw new RuntimeException("此路口无时段信息");
        }
    }

    @Override
    public AreaInfoVO areaInfo(AreaIdBO areaIdBO) {
        Integer areaId = areaIdBO.getAreaId();
        BaseAreaInfoPO baseAreaInfoPO = baseAreaInfoMapper.selectById(areaId);

        AreaInfoVO areaInfoVO = new AreaInfoVO();
        areaInfoVO.setType(baseAreaInfoPO.getType());
        areaInfoVO.setParentCode(baseAreaInfoPO.getParentCode());
        areaInfoVO.setAreaId(baseAreaInfoPO.getId());
        areaInfoVO.setAreaName(baseAreaInfoPO.getName());
        areaInfoVO.setRoadName(baseAreaInfoPO.getRoadName());
        areaInfoVO.setRemark(baseAreaInfoPO.getRemark());

        // 查询区域路口ID
        areaInfoVO.setCrossIdAndNameList(buildCrossIdAndNameList(areaId));

        String polylines = baseAreaInfoPO.getPolylines();
        // 如果是道路，直接返回线
        if (Objects.equals(AreaEnum.ROAD.getCode(), baseAreaInfoPO.getType())) {
            areaInfoVO.setWkt(polylines);
        } else {
            List<String> wktList = buildWktList(polylines);
            areaInfoVO.setWktList(wktList);
        }

        return areaInfoVO;
    }

    private List<String> buildWktList(String polylines) {
        String[] strArray = new String[0];
        if (StringUtils.isNotBlank(polylines)) {
            strArray = polylines.substring(1, polylines.length() - 1).split(", ");
        }
        return Arrays.asList(strArray);
    }

    private List<CrossIdAndNameVO> buildCrossIdAndNameList(Integer areaId) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        List<String> crossIds = areaCrossMapper.selectCrossIdsByAreaId(areaId);
        if (!CollectionUtils.isEmpty(crossIds)) {
            List<BaseCrossInfoPO> baseCrossInfoPOList = crossInfoMapper.selectByCrossIds(crossIds);
            for (BaseCrossInfoPO baseCrossInfoPO : baseCrossInfoPOList) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(baseCrossInfoPO.getId());
                crossIdAndNameVO.setCrossName(baseCrossInfoPO.getName());
                res.add(crossIdAndNameVO);
            }
        }
        return res;
    }

    private Integer findCurrentSection(String crossId) {
        Date datetime = new Date();
        String dateStr = sdf.format(datetime);
        // 获取当前计划ID
        Integer planId = findPlanId(datetime, dateStr, crossId);
        if (planId != null) {
            List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossAndPlan(crossId, planId);
            // 获取当前时间整数字面量
            SimpleDateFormat formatter = new SimpleDateFormat("HHmm");
            String timeString = formatter.format(datetime);
            int nowTimeInt = Integer.parseInt(timeString);

            for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                String startTime = crossSectionPO.getStartTime();
                String endTime = crossSectionPO.getEndTime();
                Integer startTimeInt = timeStrToInt(startTime);
                Integer endTimeInt = timeStrToInt(endTime);
                if (nowTimeInt <= endTimeInt && nowTimeInt >= startTimeInt) {
                    return crossSectionPO.getId();
                }
            }
        }
        return 0;
    }

    private Integer timeStrToInt(String time) {
        time = time.replace(":", "");
        int timeInt = Integer.parseInt(time);
        return timeInt;
    }

    public Integer findPlanId(Date datetime, String dateStr, String crossId) {
        CrossIdBO crossIdBO = new CrossIdBO();
        crossIdBO.setCrossId(crossId);
        RunningPlanDTO runningPlanDTO = runningPlanServiceImpl.listRunningPlan(crossIdBO);
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 先查询特殊日期，再查询星期
                List<String> specialDateList = execListElement.getSpecialDateList();
                List<Integer> week = execListElement.getWeek();
                if (CollectionUtil.isNotEmpty(specialDateList)) {
                    for (String specialDate : specialDateList) {
                        String substring = specialDate.substring(0, 10);
                        if (Objects.equals(dateStr, substring)) {
                            Integer planId = execListElement.getPlanId();
                            return planId;
                        }
                    }
                }
                if (CollectionUtil.isNotEmpty(week)) {
                    for (Integer day : week) {
                        // 1表示周日，2表示周一
                        int today = DateUtil.dayOfWeek(datetime);
                        today -= 1;
                        if (today == 0) {
                            today = 7;
                        }
                        if (day == today) {
                            Integer planId = execListElement.getPlanId();
                            return planId;
                        }
                    }
                }
            }
        }
        return null;
    }

    private List<AreaTreeVO> buildJuriList() {
        List<AreaTreeVO> res = new ArrayList<>();
        BaseAreaInfoPO areaInfoPO = baseAreaInfoMapper.selectCityPolice();
        AreaTreeVO areaTreeVO = new AreaTreeVO();
        areaTreeVO.setId(areaInfoPO.getId().toString());
        areaTreeVO.setName(areaInfoPO.getName());
        areaTreeVO.setType("1");
        areaTreeVO.setPolylines("");
        areaTreeVO.setPoint("");
        areaTreeVO.setChildren(buildParentPoliceList());
        res.add(areaTreeVO);
        return res;
    }

    private List<AreaTreeVO> buildSubAreaList() {
        List<AreaTreeVO> res = new ArrayList<>();
        Integer type = 4;
        return handleAreaTreeVOList(res, type);
    }

    private List<AreaTreeVO> buildRoadList() {
        List<AreaTreeVO> res = new ArrayList<>();
        Integer type = 6;
        return handleAreaTreeVOList(res, type);
    }

    private List<AreaTreeVO> handleAreaTreeVOList(List<AreaTreeVO> res, Integer type) {
        List<BaseAreaInfoPO> areaInfoPOList = baseAreaInfoMapper.selectByType(type);

        return getAreaTreeVOList(res, areaInfoPOList);
    }

    private List<AreaTreeVO> getAreaTreeVOList(List<AreaTreeVO> res, List<BaseAreaInfoPO> areaInfoPOList) {
        for (BaseAreaInfoPO baseAreaInfoPO : areaInfoPOList) {
            AreaTreeVO areaTreeVO = new AreaTreeVO();
            Integer areaId = baseAreaInfoPO.getId();
            areaTreeVO.setId(areaId.toString());
            areaTreeVO.setName(baseAreaInfoPO.getName());
            areaTreeVO.setType("1");
            String polylines = baseAreaInfoPO.getPolylines();
            // 如果是道路，直接返回线
            if (Objects.equals(AreaEnum.ROAD.getCode(), baseAreaInfoPO.getType())) {
                areaTreeVO.setPolylines(polylines);
            } else {
                List<String> wktList = buildWktList(polylines);
                areaTreeVO.setWktList(wktList);
            }
            areaTreeVO.setPoint("");
            areaTreeVO.setChildren(buildCrossList(areaId));
            res.add(areaTreeVO);
        }

        return res;
    }

    private List<AreaTreeVO> buildParentPoliceList() {
        List<AreaTreeVO> res = new ArrayList<>();
        Integer areaType = 2;
        List<BaseAreaInfoPO> areaInfoPOList = baseAreaInfoMapper.selectParentPolice(areaType);

        for (BaseAreaInfoPO baseAreaInfoPO : areaInfoPOList) {
            AreaTreeVO areaTreeVO = new AreaTreeVO();
            Integer areaId = baseAreaInfoPO.getId();
            areaTreeVO.setId(areaId.toString());
            areaTreeVO.setName(baseAreaInfoPO.getName());
            areaTreeVO.setType("1");
            String polylines = baseAreaInfoPO.getPolylines();
            // 如果是道路，直接返回线
            if (Objects.equals(AreaEnum.ROAD.getCode(), baseAreaInfoPO.getType())) {
                areaTreeVO.setPolylines(polylines);
            } else {
                List<String> wktList = buildWktList(polylines);
                areaTreeVO.setWktList(wktList);
            }
            areaTreeVO.setPoint("");
            areaTreeVO.setChildren(buildSubPoliceList(areaId));

            res.add(areaTreeVO);
        }

        return res;
    }

    private List<AreaTreeVO> buildSubPoliceList(Integer areaId) {
        List<AreaTreeVO> res = new ArrayList<>();
        List<BaseAreaInfoPO> baseAreaInfoPOList = baseAreaInfoMapper.selectByParentCode(areaId);

        return getAreaTreeVOList(res, baseAreaInfoPOList);
    }

    private List<AreaTreeVO> buildCrossList(Integer subAreaId) {
        List<AreaTreeVO> res = new ArrayList<>();

        List<String> crossIds = areaCrossMapper.selectCrossIdsByAreaId(subAreaId);
        if (CollectionUtil.isNotEmpty(crossIds)) {
            List<BaseCrossInfoPO> baseCrossInfoPOList = crossInfoMapper.selectByCrossIds(crossIds);
            for (BaseCrossInfoPO baseCrossInfoPO : baseCrossInfoPOList) {
                AreaTreeVO areaTreeVO = new AreaTreeVO();
                areaTreeVO.setId(baseCrossInfoPO.getId());
                areaTreeVO.setName(baseCrossInfoPO.getName());
                areaTreeVO.setType("2");
                areaTreeVO.setPolylines("");
                String location = baseCrossInfoPO.getLocation();
                double[] lonLat = CrossUtil.getLonLat(location);
                areaTreeVO.setPoint(lonLat[0] + "," + lonLat[1]);
                areaTreeVO.setChildren(new ArrayList<>());
                res.add(areaTreeVO);
            }
        }

        return res;
    }
}
